"""
Pydantic Models for CMDB - waf/profile

Runtime validation models for waf/profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class ProfileSignatureCustomSignatureTargetEnum(str, Enum):
    """Allowed values for target field in signature.custom-signature."""
    ARG = "arg"
    ARG_NAME = "arg-name"
    REQ_BODY = "req-body"
    REQ_COOKIE = "req-cookie"
    REQ_COOKIE_NAME = "req-cookie-name"
    REQ_FILENAME = "req-filename"
    REQ_HEADER = "req-header"
    REQ_HEADER_NAME = "req-header-name"
    REQ_RAW_URI = "req-raw-uri"
    REQ_URI = "req-uri"
    RESP_BODY = "resp-body"
    RESP_HDR = "resp-hdr"
    RESP_STATUS = "resp-status"

class ProfileMethodMethodPolicyAllowedMethodsEnum(str, Enum):
    """Allowed values for allowed_methods field in method.method-policy."""
    GET = "get"
    POST = "post"
    PUT = "put"
    HEAD = "head"
    CONNECT = "connect"
    TRACE = "trace"
    OPTIONS = "options"
    DELETE = "delete"
    OTHERS = "others"

class ProfileMethodDefaultAllowedMethodsEnum(str, Enum):
    """Allowed values for default_allowed_methods field in method."""
    GET = "get"
    POST = "post"
    PUT = "put"
    HEAD = "head"
    CONNECT = "connect"
    TRACE = "trace"
    OPTIONS = "options"
    DELETE = "delete"
    OTHERS = "others"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ProfileUrlAccessAccessPattern(BaseModel):
    """
    Child table model for url-access.access-pattern.
    
    URL access pattern.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="URL access pattern ID.")    
    srcaddr: str | None = Field(max_length=79, default=None, description="Source address.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    pattern: str = Field(max_length=511, description="URL pattern.")    
    regex: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable regular expression based pattern match.")    
    negate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable match negation.")
class ProfileUrlAccess(BaseModel):
    """
    Child table model for url-access.
    
    URL access list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="URL access ID.")    
    address: str = Field(max_length=79, description="Host address.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    action: Literal["bypass", "permit", "block"] | None = Field(default="permit", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")    
    access_pattern: list[ProfileUrlAccessAccessPattern] = Field(default_factory=list, description="URL access pattern.")
class ProfileSignatureMainClass(BaseModel):
    """
    Child table model for signature.main-class.
    
    Main signature class.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Main signature class ID.")  # datasource: ['waf.main-class.id']    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Status.")    
    action: Literal["allow", "block", "erase"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileSignatureDisabledSubClass(BaseModel):
    """
    Child table model for signature.disabled-sub-class.
    
    Disabled signature subclasses.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Signature subclass ID.")  # datasource: ['waf.sub-class.id']
class ProfileSignatureDisabledSignature(BaseModel):
    """
    Child table model for signature.disabled-signature.
    
    Disabled signatures.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Signature ID.")  # datasource: ['waf.signature.id']
class ProfileSignatureCustomSignature(BaseModel):
    """
    Child table model for signature.custom-signature.
    
    Custom signature.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=35, default=None, description="Signature name.")    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Status.")    
    action: Literal["allow", "block", "erase"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")    
    direction: Literal["request", "response"] | None = Field(default="request", description="Traffic direction.")    
    case_sensitivity: Literal["disable", "enable"] | None = Field(default="disable", description="Case sensitivity in pattern.")    
    pattern: str | None = Field(max_length=511, default=None, description="Match pattern.")    
    target: list[ProfileSignatureCustomSignatureTargetEnum] = Field(default_factory=list, description="Match HTTP target.")
class ProfileSignature(BaseModel):
    """
    Child table model for signature.
    
    WAF signatures.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    main_class: list[ProfileSignatureMainClass] = Field(default_factory=list, description="Main signature class.")    
    disabled_sub_class: list[ProfileSignatureDisabledSubClass] = Field(default_factory=list, description="Disabled signature subclasses.")    
    disabled_signature: list[ProfileSignatureDisabledSignature] = Field(default_factory=list, description="Disabled signatures.")    
    credit_card_detection_threshold: int | None = Field(ge=0, le=128, default=3, description="The minimum number of Credit cards to detect violation.")    
    custom_signature: list[ProfileSignatureCustomSignature] = Field(default_factory=list, description="Custom signature.")
class ProfileMethodMethodPolicy(BaseModel):
    """
    Child table model for method.method-policy.
    
    HTTP method policy.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="HTTP method policy ID.")    
    pattern: str | None = Field(max_length=511, default=None, description="URL pattern.")    
    regex: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable regular expression based pattern match.")    
    address: str = Field(max_length=79, description="Host address.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    allowed_methods: list[ProfileMethodMethodPolicyAllowedMethodsEnum] = Field(default_factory=list, description="Allowed Methods.")
class ProfileMethod(BaseModel):
    """
    Child table model for method.
    
    Method restriction.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Status.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")    
    default_allowed_methods: list[ProfileMethodDefaultAllowedMethodsEnum] = Field(default_factory=list, description="Methods.")    
    method_policy: list[ProfileMethodMethodPolicy] = Field(default_factory=list, description="HTTP method policy.")
class ProfileConstraintVersion(BaseModel):
    """
    Child table model for constraint.version.
    
    Enable/disable HTTP version check.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintUrlParamLength(BaseModel):
    """
    Child table model for constraint.url-param-length.
    
    Maximum length of parameter in URL.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    length: int | None = Field(ge=0, le=2147483647, default=8192, description="Maximum length of URL parameter in bytes (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintParamLength(BaseModel):
    """
    Child table model for constraint.param-length.
    
    Maximum length of parameter in URL, HTTP POST request or HTTP body.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    length: int | None = Field(ge=0, le=2147483647, default=8192, description="Maximum length of parameter in URL, HTTP POST request or HTTP body in bytes (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMethod(BaseModel):
    """
    Child table model for constraint.method.
    
    Enable/disable HTTP method check.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMaxUrlParam(BaseModel):
    """
    Child table model for constraint.max-url-param.
    
    Maximum number of parameters in URL.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    max_url_param: int | None = Field(ge=0, le=2147483647, default=16, description="Maximum number of parameters in URL (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMaxRangeSegment(BaseModel):
    """
    Child table model for constraint.max-range-segment.
    
    Maximum number of range segments in HTTP range line.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    max_range_segment: int | None = Field(ge=0, le=2147483647, default=5, description="Maximum number of range segments in HTTP range line (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMaxHeaderLine(BaseModel):
    """
    Child table model for constraint.max-header-line.
    
    Maximum number of HTTP header line.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    max_header_line: int | None = Field(ge=0, le=2147483647, default=32, description="Maximum number HTTP header lines (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMaxCookie(BaseModel):
    """
    Child table model for constraint.max-cookie.
    
    Maximum number of cookies in HTTP request.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    max_cookie: int | None = Field(ge=0, le=2147483647, default=16, description="Maximum number of cookies in HTTP request (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintMalformed(BaseModel):
    """
    Child table model for constraint.malformed.
    
    Enable/disable malformed HTTP request check.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintLineLength(BaseModel):
    """
    Child table model for constraint.line-length.
    
    HTTP line length in request.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    length: int | None = Field(ge=0, le=2147483647, default=1024, description="Length of HTTP line in bytes (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintHostname(BaseModel):
    """
    Child table model for constraint.hostname.
    
    Enable/disable hostname check.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintHeaderLength(BaseModel):
    """
    Child table model for constraint.header-length.
    
    HTTP header length in request.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    length: int | None = Field(ge=0, le=2147483647, default=8192, description="Length of HTTP header in bytes (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraintException(BaseModel):
    """
    Child table model for constraint.exception.
    
    HTTP constraint exception.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Exception ID.")    
    pattern: str = Field(max_length=511, description="URL pattern.")    
    regex: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable regular expression based pattern match.")    
    address: str = Field(max_length=79, description="Host address.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    header_length: Literal["enable", "disable"] | None = Field(default="disable", description="HTTP header length in request.")    
    content_length: Literal["enable", "disable"] | None = Field(default="disable", description="HTTP content length in request.")    
    param_length: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum length of parameter in URL, HTTP POST request or HTTP body.")    
    line_length: Literal["enable", "disable"] | None = Field(default="disable", description="HTTP line length in request.")    
    url_param_length: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum length of parameter in URL.")    
    version: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable HTTP version check.")    
    method: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable HTTP method check.")    
    hostname: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable hostname check.")    
    malformed: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable malformed HTTP request check.")    
    max_cookie: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum number of cookies in HTTP request.")    
    max_header_line: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum number of HTTP header line.")    
    max_url_param: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum number of parameters in URL.")    
    max_range_segment: Literal["enable", "disable"] | None = Field(default="disable", description="Maximum number of range segments in HTTP range line.")
class ProfileConstraintContentLength(BaseModel):
    """
    Child table model for constraint.content-length.
    
    HTTP content length in request.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the constraint.")    
    length: int | None = Field(ge=0, le=2147483647, default=67108864, description="Length of HTTP content in bytes (0 to 2147483647).")    
    action: Literal["allow", "block"] | None = Field(default="allow", description="Action.")    
    log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")
class ProfileConstraint(BaseModel):
    """
    Child table model for constraint.
    
    WAF HTTP protocol restrictions.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    header_length: ProfileConstraintHeaderLength | None = Field(default=None, description="HTTP header length in request.")    
    content_length: ProfileConstraintContentLength | None = Field(default=None, description="HTTP content length in request.")    
    param_length: ProfileConstraintParamLength | None = Field(default=None, description="Maximum length of parameter in URL, HTTP POST request or HTTP body.")    
    line_length: ProfileConstraintLineLength | None = Field(default=None, description="HTTP line length in request.")    
    url_param_length: ProfileConstraintUrlParamLength | None = Field(default=None, description="Maximum length of parameter in URL.")    
    version: ProfileConstraintVersion | None = Field(default=None, description="Enable/disable HTTP version check.")    
    method: ProfileConstraintMethod | None = Field(default=None, description="Enable/disable HTTP method check.")    
    hostname: ProfileConstraintHostname | None = Field(default=None, description="Enable/disable hostname check.")    
    malformed: ProfileConstraintMalformed | None = Field(default=None, description="Enable/disable malformed HTTP request check.")    
    max_cookie: ProfileConstraintMaxCookie | None = Field(default=None, description="Maximum number of cookies in HTTP request.")    
    max_header_line: ProfileConstraintMaxHeaderLine | None = Field(default=None, description="Maximum number of HTTP header line.")    
    max_url_param: ProfileConstraintMaxUrlParam | None = Field(default=None, description="Maximum number of parameters in URL.")    
    max_range_segment: ProfileConstraintMaxRangeSegment | None = Field(default=None, description="Maximum number of range segments in HTTP range line.")    
    exception: list[ProfileConstraintException] = Field(default_factory=list, description="HTTP constraint exception.")
class ProfileAddressListTrustedAddress(BaseModel):
    """
    Child table model for address-list.trusted-address.
    
    Trusted address.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Address name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
class ProfileAddressListBlockedAddress(BaseModel):
    """
    Child table model for address-list.blocked-address.
    
    Blocked address.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Address name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
class ProfileAddressList(BaseModel):
    """
    Child table model for address-list.
    
    Address block and allow lists.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Status.")    
    blocked_log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging on blocked addresses.")    
    severity: Literal["high", "medium", "low"] | None = Field(default="medium", description="Severity.")    
    trusted_address: list[ProfileAddressListTrustedAddress] = Field(default_factory=list, description="Trusted address.")    
    blocked_address: list[ProfileAddressListBlockedAddress] = Field(default_factory=list, description="Blocked address.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ProfileModel(BaseModel):
    """
    Pydantic model for waf/profile configuration.
    
    Configure Web application firewall configuration.
    
    Validation Rules:        - name: max_length=47 pattern=        - external: pattern=        - extended_log: pattern=        - signature: pattern=        - constraint: pattern=        - method: pattern=        - address_list: pattern=        - url_access: pattern=        - comment: max_length=1023 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=47, default=None, description="WAF Profile name.")    
    external: Literal["disable", "enable"] | None = Field(default="disable", description="Disable/Enable external HTTP Inspection.")    
    extended_log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable extended logging.")    
    signature: ProfileSignature | None = Field(default=None, description="WAF signatures.")    
    constraint: ProfileConstraint | None = Field(default=None, description="WAF HTTP protocol restrictions.")    
    method: ProfileMethod | None = Field(default=None, description="Method restriction.")    
    address_list: ProfileAddressList | None = Field(default=None, description="Address block and allow lists.")    
    url_access: list[ProfileUrlAccess] = Field(default_factory=list, description="URL access list.")    
    comment: str | None = Field(max_length=1023, default=None, description="Comment.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_url_access_references(self, client: Any) -> list[str]:
        """
        Validate url_access references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     url_access=[{"address": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_url_access_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.waf.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "url_access", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("address")
            else:
                value = getattr(item, "address", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.address.exists(value):
                found = True
            elif await client.api.cmdb.firewall.addrgrp.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Url-Access '{value}' not found in "
                    "firewall/address or firewall/addrgrp"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_url_access_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ProfileModel",    "ProfileSignature",    "ProfileSignature.MainClass",    "ProfileSignature.DisabledSubClass",    "ProfileSignature.DisabledSignature",    "ProfileSignature.CustomSignature",    "ProfileConstraint",    "ProfileConstraint.HeaderLength",    "ProfileConstraint.ContentLength",    "ProfileConstraint.ParamLength",    "ProfileConstraint.LineLength",    "ProfileConstraint.UrlParamLength",    "ProfileConstraint.Version",    "ProfileConstraint.Method",    "ProfileConstraint.Hostname",    "ProfileConstraint.Malformed",    "ProfileConstraint.MaxCookie",    "ProfileConstraint.MaxHeaderLine",    "ProfileConstraint.MaxUrlParam",    "ProfileConstraint.MaxRangeSegment",    "ProfileConstraint.Exception",    "ProfileMethod",    "ProfileMethod.MethodPolicy",    "ProfileAddressList",    "ProfileAddressList.TrustedAddress",    "ProfileAddressList.BlockedAddress",    "ProfileUrlAccess",    "ProfileUrlAccess.AccessPattern",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:23:59.256477Z
# ============================================================================


"""FortiOS CMDB - ExtensionController category"""

from .fortigate import Fortigate
from .lan_extension_vdom_status import LanExtensionVdomStatus

__all__ = [
    "ExtensionController",
    "Fortigate",
    "LanExtensionVdomStatus",
]


class ExtensionController:
    """ExtensionController endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """ExtensionController endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.fortigate = Fortigate(client)
        self.lan_extension_vdom_status = LanExtensionVdomStatus(client)



"""
Pydantic Models for CMDB - ips/sensor

Runtime validation models for ips/sensor configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class SensorEntriesActionEnum(str, Enum):
    """Allowed values for action field in entries."""
    PASS = "pass"
    BLOCK = "block"
    RESET = "reset"
    DEFAULT = "default"

class SensorEntriesRateTrackEnum(str, Enum):
    """Allowed values for rate_track field in entries."""
    NONE = "none"
    SRC_IP = "src-ip"
    DEST_IP = "dest-ip"
    DHCP_CLIENT_MAC = "dhcp-client-mac"
    DNS_DOMAIN = "dns-domain"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SensorEntriesVulnType(BaseModel):
    """
    Child table model for entries.vuln-type.
    
    List of signature vulnerability types to filter by.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Vulnerability type ID.")
class SensorEntriesRule(BaseModel):
    """
    Child table model for entries.rule.
    
    Identifies the predefined or custom IPS signatures to add to the sensor.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Rule IPS.")
class SensorEntriesExemptIp(BaseModel):
    """
    Child table model for entries.exempt-ip.
    
    Traffic from selected source or destination IP addresses is exempt from this signature.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Exempt IP ID.")    
    src_ip: str | None = Field(default="0.0.0.0 0.0.0.0", description="Source IP address and netmask (applies to packet matching the signature).")    
    dst_ip: str | None = Field(default="0.0.0.0 0.0.0.0", description="Destination IP address and netmask (applies to packet matching the signature).")
class SensorEntriesCve(BaseModel):
    """
    Child table model for entries.cve.
    
    List of CVE IDs of the signatures to add to the sensor.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    cve_entry: str = Field(max_length=19, description="CVE IDs or CVE wildcards.")
class SensorEntries(BaseModel):
    """
    Child table model for entries.
    
    IPS sensor filter.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Rule ID in IPS database (0 - 4294967295).")    
    rule: list[SensorEntriesRule] = Field(default_factory=list, description="Identifies the predefined or custom IPS signatures to add to the sensor.")    
    location: list[str] = Field(default_factory=list, description="Protect client or server traffic.")    
    severity: list[str] = Field(default_factory=list, description="Relative severity of the signature, from info to critical. Log messages generated by the signature include the severity.")    
    protocol: list[str] = Field(default_factory=list, description="Protocols to be examined. Use all for every protocol and other for unlisted protocols.")    
    os: list[str] = Field(default_factory=list, description="Operating systems to be protected. Use all for every operating system and other for unlisted operating systems.")    
    application: list[str] = Field(default_factory=list, description="Operating systems to be protected. Use all for every application and other for unlisted application.")    
    default_action: Literal["all", "pass", "block"] | None = Field(default="all", description="Signature default action filter.")    
    default_status: Literal["all", "enable", "disable"] | None = Field(default="all", description="Signature default status filter.")    
    cve: list[SensorEntriesCve] = Field(default_factory=list, description="List of CVE IDs of the signatures to add to the sensor.")    
    vuln_type: list[SensorEntriesVulnType] = Field(default_factory=list, description="List of signature vulnerability types to filter by.")    
    last_modified: str | None = Field(default=None, description="Filter by signature last modified date. Formats: before <date>, after <date>, between <start-date> <end-date>.")    
    status: Literal["disable", "enable", "default"] | None = Field(default="default", description="Status of the signatures included in filter. Only those filters with a status to enable are used.")    
    log: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable logging of signatures included in filter.")    
    log_packet: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable packet logging. Enable to save the packet that triggers the filter. You can download the packets in pcap format for diagnostic use.")    
    log_attack_context: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of attack context: URL buffer, header buffer, body buffer, packet buffer.")    
    action: SensorEntriesActionEnum | None = Field(default=SensorEntriesActionEnum.DEFAULT, description="Action taken with traffic in which signatures are detected.")    
    rate_count: int | None = Field(ge=0, le=65535, default=0, description="Count of the rate.")    
    rate_duration: int | None = Field(ge=1, le=65535, default=60, description="Duration (sec) of the rate.")    
    rate_mode: Literal["periodical", "continuous"] | None = Field(default="continuous", description="Rate limit mode.")    
    rate_track: SensorEntriesRateTrackEnum | None = Field(default=SensorEntriesRateTrackEnum.NONE, description="Track the packet protocol field.")    
    exempt_ip: list[SensorEntriesExemptIp] = Field(default_factory=list, description="Traffic from selected source or destination IP addresses is exempt from this signature.")    
    quarantine: Literal["none", "attacker"] | None = Field(default="none", description="Quarantine method.")    
    quarantine_expiry: str | None = Field(default="5m", description="Duration of quarantine. (Format ###d##h##m, minimum 1m, maximum 364d23h59m, default = 5m). Requires quarantine set to attacker.")    
    quarantine_log: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable quarantine logging.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SensorModel(BaseModel):
    """
    Pydantic model for ips/sensor configuration.
    
    Configure IPS sensor.
    
    Validation Rules:        - name: max_length=47 pattern=        - comment: max_length=255 pattern=        - replacemsg_group: max_length=35 pattern=        - block_malicious_url: pattern=        - scan_botnet_connections: pattern=        - extended_log: pattern=        - entries: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=47, description="Sensor name.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")    
    replacemsg_group: str | None = Field(max_length=35, default=None, description="Replacement message group.")  # datasource: ['system.replacemsg-group.name']    
    block_malicious_url: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable malicious URL blocking.")    
    scan_botnet_connections: Literal["disable", "block", "monitor"] | None = Field(default="disable", description="Block or monitor connections to Botnet servers, or disable Botnet scanning.")    
    extended_log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable extended logging.")    
    entries: list[SensorEntries] = Field(default_factory=list, description="IPS sensor filter.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('replacemsg_group')
    @classmethod
    def validate_replacemsg_group(cls, v: Any) -> Any:
        """
        Validate replacemsg_group field.
        
        Datasource: ['system.replacemsg-group.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SensorModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_replacemsg_group_references(self, client: Any) -> list[str]:
        """
        Validate replacemsg_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/replacemsg-group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SensorModel(
            ...     replacemsg_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_replacemsg_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ips.sensor.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "replacemsg_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.replacemsg_group.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Replacemsg-Group '{value}' not found in "
                "system/replacemsg-group"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_replacemsg_group_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SensorModel",    "SensorEntries",    "SensorEntries.Rule",    "SensorEntries.Cve",    "SensorEntries.VulnType",    "SensorEntries.ExemptIp",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:23:59.837624Z
# ============================================================================


"""FortiOS CMDB - SaasApplication category"""

from .details import Details

__all__ = [
    "Details",
    "SaasApplication",
]


class SaasApplication:
    """SaasApplication endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """SaasApplication endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.details = Details(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/local_in_policy6
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class LocalInPolicy6IntfItem(TypedDict, total=False):
    """Nested item for intf field."""
    name: str


class LocalInPolicy6SrcaddrItem(TypedDict, total=False):
    """Nested item for srcaddr field."""
    name: str


class LocalInPolicy6DstaddrItem(TypedDict, total=False):
    """Nested item for dstaddr field."""
    name: str


class LocalInPolicy6Internetservice6srcnameItem(TypedDict, total=False):
    """Nested item for internet-service6-src-name field."""
    name: str


class LocalInPolicy6Internetservice6srcgroupItem(TypedDict, total=False):
    """Nested item for internet-service6-src-group field."""
    name: str


class LocalInPolicy6Internetservice6srccustomItem(TypedDict, total=False):
    """Nested item for internet-service6-src-custom field."""
    name: str


class LocalInPolicy6Internetservice6srccustomgroupItem(TypedDict, total=False):
    """Nested item for internet-service6-src-custom-group field."""
    name: str


class LocalInPolicy6Internetservice6srcfortiguardItem(TypedDict, total=False):
    """Nested item for internet-service6-src-fortiguard field."""
    name: str


class LocalInPolicy6ServiceItem(TypedDict, total=False):
    """Nested item for service field."""
    name: str


class LocalInPolicy6Payload(TypedDict, total=False):
    """Payload type for LocalInPolicy6 operations."""
    policyid: int
    uuid: str
    intf: str | list[str] | list[LocalInPolicy6IntfItem]
    srcaddr: str | list[str] | list[LocalInPolicy6SrcaddrItem]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: str | list[str] | list[LocalInPolicy6DstaddrItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: str | list[str] | list[LocalInPolicy6Internetservice6srcnameItem]
    internet_service6_src_group: str | list[str] | list[LocalInPolicy6Internetservice6srcgroupItem]
    internet_service6_src_custom: str | list[str] | list[LocalInPolicy6Internetservice6srccustomItem]
    internet_service6_src_custom_group: str | list[str] | list[LocalInPolicy6Internetservice6srccustomgroupItem]
    internet_service6_src_fortiguard: str | list[str] | list[LocalInPolicy6Internetservice6srcfortiguardItem]
    dstaddr_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    service: str | list[str] | list[LocalInPolicy6ServiceItem]
    service_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    schedule: str
    status: Literal["enable", "disable"]
    virtual_patch: Literal["enable", "disable"]
    logtraffic: Literal["enable", "disable"]
    comments: str


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class LocalInPolicy6Response(TypedDict, total=False):
    """Response type for LocalInPolicy6 - use with .dict property for typed dict access."""
    policyid: int
    uuid: str
    intf: list[LocalInPolicy6IntfItem]
    srcaddr: list[LocalInPolicy6SrcaddrItem]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: list[LocalInPolicy6DstaddrItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: list[LocalInPolicy6Internetservice6srcnameItem]
    internet_service6_src_group: list[LocalInPolicy6Internetservice6srcgroupItem]
    internet_service6_src_custom: list[LocalInPolicy6Internetservice6srccustomItem]
    internet_service6_src_custom_group: list[LocalInPolicy6Internetservice6srccustomgroupItem]
    internet_service6_src_fortiguard: list[LocalInPolicy6Internetservice6srcfortiguardItem]
    dstaddr_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    service: list[LocalInPolicy6ServiceItem]
    service_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    schedule: str
    status: Literal["enable", "disable"]
    virtual_patch: Literal["enable", "disable"]
    logtraffic: Literal["enable", "disable"]
    comments: str


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class LocalInPolicy6IntfItemObject(FortiObject[LocalInPolicy6IntfItem]):
    """Typed object for intf table items with attribute access."""
    name: str


class LocalInPolicy6SrcaddrItemObject(FortiObject[LocalInPolicy6SrcaddrItem]):
    """Typed object for srcaddr table items with attribute access."""
    name: str


class LocalInPolicy6DstaddrItemObject(FortiObject[LocalInPolicy6DstaddrItem]):
    """Typed object for dstaddr table items with attribute access."""
    name: str


class LocalInPolicy6Internetservice6srcnameItemObject(FortiObject[LocalInPolicy6Internetservice6srcnameItem]):
    """Typed object for internet-service6-src-name table items with attribute access."""
    name: str


class LocalInPolicy6Internetservice6srcgroupItemObject(FortiObject[LocalInPolicy6Internetservice6srcgroupItem]):
    """Typed object for internet-service6-src-group table items with attribute access."""
    name: str


class LocalInPolicy6Internetservice6srccustomItemObject(FortiObject[LocalInPolicy6Internetservice6srccustomItem]):
    """Typed object for internet-service6-src-custom table items with attribute access."""
    name: str


class LocalInPolicy6Internetservice6srccustomgroupItemObject(FortiObject[LocalInPolicy6Internetservice6srccustomgroupItem]):
    """Typed object for internet-service6-src-custom-group table items with attribute access."""
    name: str


class LocalInPolicy6Internetservice6srcfortiguardItemObject(FortiObject[LocalInPolicy6Internetservice6srcfortiguardItem]):
    """Typed object for internet-service6-src-fortiguard table items with attribute access."""
    name: str


class LocalInPolicy6ServiceItemObject(FortiObject[LocalInPolicy6ServiceItem]):
    """Typed object for service table items with attribute access."""
    name: str


class LocalInPolicy6Object(FortiObject):
    """Typed FortiObject for LocalInPolicy6 with field access."""
    policyid: int
    uuid: str
    intf: FortiObjectList[LocalInPolicy6IntfItemObject]
    srcaddr: FortiObjectList[LocalInPolicy6SrcaddrItemObject]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: FortiObjectList[LocalInPolicy6DstaddrItemObject]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: FortiObjectList[LocalInPolicy6Internetservice6srcnameItemObject]
    internet_service6_src_group: FortiObjectList[LocalInPolicy6Internetservice6srcgroupItemObject]
    internet_service6_src_custom: FortiObjectList[LocalInPolicy6Internetservice6srccustomItemObject]
    internet_service6_src_custom_group: FortiObjectList[LocalInPolicy6Internetservice6srccustomgroupItemObject]
    internet_service6_src_fortiguard: FortiObjectList[LocalInPolicy6Internetservice6srcfortiguardItemObject]
    dstaddr_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    service: FortiObjectList[LocalInPolicy6ServiceItemObject]
    service_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    schedule: str
    status: Literal["enable", "disable"]
    virtual_patch: Literal["enable", "disable"]
    logtraffic: Literal["enable", "disable"]
    comments: str


# ================================================================
# Main Endpoint Class
# ================================================================

class LocalInPolicy6:
    """
    
    Endpoint: firewall/local_in_policy6
    Category: cmdb
    MKey: policyid
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        policyid: int,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> LocalInPolicy6Object: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[LocalInPolicy6Object]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: LocalInPolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        intf: str | list[str] | list[LocalInPolicy6IntfItem] | None = ...,
        srcaddr: str | list[str] | list[LocalInPolicy6SrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[LocalInPolicy6DstaddrItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[LocalInPolicy6Internetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[LocalInPolicy6Internetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[LocalInPolicy6Internetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[LocalInPolicy6Internetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[LocalInPolicy6Internetservice6srcfortiguardItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        service: str | list[str] | list[LocalInPolicy6ServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        virtual_patch: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> LocalInPolicy6Object: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: LocalInPolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        intf: str | list[str] | list[LocalInPolicy6IntfItem] | None = ...,
        srcaddr: str | list[str] | list[LocalInPolicy6SrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[LocalInPolicy6DstaddrItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[LocalInPolicy6Internetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[LocalInPolicy6Internetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[LocalInPolicy6Internetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[LocalInPolicy6Internetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[LocalInPolicy6Internetservice6srcfortiguardItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        service: str | list[str] | list[LocalInPolicy6ServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        virtual_patch: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> LocalInPolicy6Object: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: LocalInPolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        intf: str | list[str] | list[LocalInPolicy6IntfItem] | None = ...,
        srcaddr: str | list[str] | list[LocalInPolicy6SrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[LocalInPolicy6DstaddrItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[LocalInPolicy6Internetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[LocalInPolicy6Internetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[LocalInPolicy6Internetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[LocalInPolicy6Internetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[LocalInPolicy6Internetservice6srcfortiguardItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        service: str | list[str] | list[LocalInPolicy6ServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        virtual_patch: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "LocalInPolicy6",
    "LocalInPolicy6Payload",
    "LocalInPolicy6Response",
    "LocalInPolicy6Object",
]


"""
Pydantic Models for CMDB - router/multicast

Runtime validation models for router/multicast configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class MulticastPimSmGlobalRpAddress(BaseModel):
    """
    Child table model for pim-sm-global.rp-address.
    
    Statically configure RP addresses.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    ip_address: str = Field(default="0.0.0.0", description="RP router address.")    
    group: str | None = Field(max_length=35, default=None, description="Groups to use this RP.")  # datasource: ['router.access-list.name']
class MulticastPimSmGlobalVrfRpAddress(BaseModel):
    """
    Child table model for pim-sm-global-vrf.rp-address.
    
    Statically configure RP addresses.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    ip_address: str = Field(default="0.0.0.0", description="RP router address.")    
    group: str | None = Field(max_length=35, default=None, description="Groups to use this RP.")  # datasource: ['router.access-list.name']
class MulticastPimSmGlobalVrf(BaseModel):
    """
    Child table model for pim-sm-global-vrf.
    
    per-VRF PIM sparse-mode global settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    vrf: int | None = Field(ge=1, le=511, default=0, description="VRF ID.")    
    bsr_candidate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable allowing this router to become a bootstrap router (BSR).")    
    bsr_interface: str | None = Field(max_length=15, default=None, description="Interface to advertise as candidate BSR.")  # datasource: ['system.interface.name']    
    bsr_priority: int | None = Field(ge=0, le=255, default=0, description="BSR priority (0 - 255, default = 0).")    
    bsr_hash: int | None = Field(ge=0, le=32, default=10, description="BSR hash length (0 - 32, default = 10).")    
    bsr_allow_quick_refresh: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable accept BSR quick refresh packets from neighbors.")    
    cisco_crp_prefix: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable making candidate RP compatible with old Cisco IOS.")    
    rp_address: list[MulticastPimSmGlobalVrfRpAddress] = Field(default_factory=list, description="Statically configure RP addresses.")
class MulticastPimSmGlobal(BaseModel):
    """
    Child table model for pim-sm-global.
    
    PIM sparse-mode global settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    message_interval: int | None = Field(ge=1, le=65535, default=60, description="Period of time between sending periodic PIM join/prune messages in seconds (1 - 65535, default = 60).")    
    join_prune_holdtime: int | None = Field(ge=1, le=65535, default=210, description="Join/prune holdtime (1 - 65535, default = 210).")    
    accept_register_list: str | None = Field(max_length=35, default=None, description="Sources allowed to register packets with this Rendezvous Point (RP).")  # datasource: ['router.access-list.name']    
    accept_source_list: str | None = Field(max_length=35, default=None, description="Sources allowed to send multicast traffic.")  # datasource: ['router.access-list.name']    
    bsr_candidate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable allowing this router to become a bootstrap router (BSR).")    
    bsr_interface: str | None = Field(max_length=15, default=None, description="Interface to advertise as candidate BSR.")  # datasource: ['system.interface.name']    
    bsr_priority: int | None = Field(ge=0, le=255, default=0, description="BSR priority (0 - 255, default = 0).")    
    bsr_hash: int | None = Field(ge=0, le=32, default=10, description="BSR hash length (0 - 32, default = 10).")    
    bsr_allow_quick_refresh: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable accept BSR quick refresh packets from neighbors.")    
    cisco_crp_prefix: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable making candidate RP compatible with old Cisco IOS.")    
    cisco_register_checksum: Literal["enable", "disable"] | None = Field(default="disable", description="Checksum entire register packet(for old Cisco IOS compatibility).")    
    cisco_register_checksum_group: str | None = Field(max_length=35, default=None, description="Cisco register checksum only these groups.")  # datasource: ['router.access-list.name']    
    cisco_ignore_rp_set_priority: Literal["enable", "disable"] | None = Field(default="disable", description="Use only hash for RP selection (compatibility with old Cisco IOS).")    
    register_rp_reachability: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable check RP is reachable before registering packets.")    
    register_source: Literal["disable", "interface", "ip-address"] | None = Field(default="disable", description="Override source address in register packets.")    
    register_source_interface: str | None = Field(max_length=15, default=None, description="Override with primary interface address.")  # datasource: ['system.interface.name']    
    register_source_ip: str | None = Field(default="0.0.0.0", description="Override with local IP address.")    
    register_supression: int | None = Field(ge=1, le=65535, default=60, description="Period of time to honor register-stop message (1 - 65535 sec, default = 60).")    
    null_register_retries: int | None = Field(ge=1, le=20, default=1, description="Maximum retries of null register (1 - 20, default = 1).")    
    rp_register_keepalive: int | None = Field(ge=1, le=65535, default=185, description="Timeout for RP receiving data on (S,G) tree (1 - 65535 sec, default = 185).")    
    spt_threshold: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable switching to source specific trees.")    
    spt_threshold_group: str | None = Field(max_length=35, default=None, description="Groups allowed to switch to source tree.")  # datasource: ['router.access-list.name']    
    ssm: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable source specific multicast.")    
    ssm_range: str | None = Field(max_length=35, default=None, description="Groups allowed to source specific multicast.")  # datasource: ['router.access-list.name']    
    register_rate_limit: int | None = Field(ge=0, le=65535, default=0, description="Limit of packets/sec per source registered through this RP (0 - 65535, default = 0 which means unlimited).")    
    pim_use_sdwan: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of SDWAN when checking RPF neighbor and sending of REG packet.")    
    rp_address: list[MulticastPimSmGlobalRpAddress] = Field(default_factory=list, description="Statically configure RP addresses.")
class MulticastInterfaceJoinGroup(BaseModel):
    """
    Child table model for interface.join-group.
    
    Join multicast groups.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    address: str | None = Field(default="0.0.0.0", description="Multicast group IP address.")
class MulticastInterfaceIgmp(BaseModel):
    """
    Child table model for interface.igmp.
    
    IGMP configuration options.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    access_group: str | None = Field(max_length=35, default=None, description="Groups IGMP hosts are allowed to join.")  # datasource: ['router.access-list.name']    
    version: Literal["3", "2", "1"] | None = Field(default="3", description="Maximum version of IGMP to support.")    
    immediate_leave_group: str | None = Field(max_length=35, default=None, description="Groups to drop membership for immediately after receiving IGMPv2 leave.")  # datasource: ['router.access-list.name']    
    last_member_query_interval: int | None = Field(ge=1, le=65535, default=1000, description="Timeout between IGMPv2 leave and removing group (1 - 65535 msec, default = 1000).")    
    last_member_query_count: int | None = Field(ge=2, le=7, default=2, description="Number of group specific queries before removing group (2 - 7, default = 2).")    
    query_max_response_time: int | None = Field(ge=1, le=25, default=10, description="Maximum time to wait for a IGMP query response (1 - 25 sec, default = 10).")    
    query_interval: int | None = Field(ge=1, le=65535, default=125, description="Interval between queries to IGMP hosts (1 - 65535 sec, default = 125).")    
    query_timeout: int | None = Field(ge=60, le=900, default=255, description="Timeout between queries before becoming querying unit for network (60 - 900, default = 255).")    
    router_alert_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable require IGMP packets contain router alert option.")
class MulticastInterface(BaseModel):
    """
    Child table model for interface.
    
    PIM interfaces.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=15, default=None, description="Interface name.")  # datasource: ['system.interface.name']    
    ttl_threshold: int | None = Field(ge=1, le=255, default=1, description="Minimum TTL of multicast packets that will be forwarded (applied only to new multicast routes) (1 - 255, default = 1).")    
    pim_mode: Literal["sparse-mode", "dense-mode"] | None = Field(default="sparse-mode", description="PIM operation mode.")    
    passive: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable listening to IGMP but not participating in PIM.")    
    bfd: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Protocol Independent Multicast (PIM) Bidirectional Forwarding Detection (BFD).")    
    neighbour_filter: str | None = Field(max_length=35, default=None, description="Routers acknowledged as neighbor routers.")  # datasource: ['router.access-list.name']    
    hello_interval: int | None = Field(ge=1, le=65535, default=30, description="Interval between sending PIM hello messages (0 - 65535 sec, default = 30).")    
    hello_holdtime: int | None = Field(ge=1, le=65535, default=105, description="Time before old neighbor information expires (0 - 65535 sec, default = 105).")    
    cisco_exclude_genid: Literal["enable", "disable"] | None = Field(default="disable", description="Exclude GenID from hello packets (compatibility with old Cisco IOS).")    
    dr_priority: int | None = Field(ge=1, le=4294967295, default=1, description="DR election priority.")    
    propagation_delay: int | None = Field(ge=100, le=5000, default=500, description="Delay flooding packets on this interface (100 - 5000 msec, default = 500).")    
    state_refresh_interval: int | None = Field(ge=1, le=100, default=60, description="Interval between sending state-refresh packets (1 - 100 sec, default = 60).")    
    rp_candidate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable compete to become RP in elections.")    
    rp_candidate_group: str | None = Field(max_length=35, default=None, description="Multicast groups managed by this RP.")  # datasource: ['router.access-list.name']    
    rp_candidate_priority: int | None = Field(ge=0, le=255, default=192, description="Router's priority as RP.")    
    rp_candidate_interval: int | None = Field(ge=1, le=16383, default=60, description="RP candidate advertisement interval (1 - 16383 sec, default = 60).")    
    multicast_flow: str | None = Field(max_length=35, default=None, description="Acceptable source for multicast group.")  # datasource: ['router.multicast-flow.name']    
    static_group: str | None = Field(max_length=35, default=None, description="Statically set multicast groups to forward out.")  # datasource: ['router.multicast-flow.name']    
    rpf_nbr_fail_back: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable fail back for RPF neighbor query.")    
    rpf_nbr_fail_back_filter: str | None = Field(max_length=35, default=None, description="Filter for fail back RPF neighbors.")  # datasource: ['router.access-list.name']    
    join_group: list[MulticastInterfaceJoinGroup] = Field(default_factory=list, description="Join multicast groups.")    
    igmp: MulticastInterfaceIgmp | None = Field(default=None, description="IGMP configuration options.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class MulticastModel(BaseModel):
    """
    Pydantic model for router/multicast configuration.
    
    Configure router multicast.
    
    Validation Rules:        - route_threshold: min=1 max=2147483647 pattern=        - route_limit: min=1 max=2147483647 pattern=        - multicast_routing: pattern=        - pim_sm_global: pattern=        - pim_sm_global_vrf: pattern=        - interface: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    route_threshold: int | None = Field(ge=1, le=2147483647, default=None, description="Generate warnings when the number of multicast routes exceeds this number, must not be greater than route-limit.")    
    route_limit: int | None = Field(ge=1, le=2147483647, default=2147483647, description="Maximum number of multicast routes.")    
    multicast_routing: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IP multicast routing.")    
    pim_sm_global: MulticastPimSmGlobal | None = Field(default=None, description="PIM sparse-mode global settings.")    
    pim_sm_global_vrf: list[MulticastPimSmGlobalVrf] = Field(default_factory=list, description="per-VRF PIM sparse-mode global settings.")    
    interface: list[MulticastInterface] = Field(default_factory=list, description="PIM interfaces.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "MulticastModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_pim_sm_global_references(self, client: Any) -> list[str]:
        """
        Validate pim_sm_global references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/access-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = MulticastModel(
            ...     pim_sm_global=[{"ssm-range": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_pim_sm_global_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.multicast.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "pim_sm_global", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("ssm-range")
            else:
                value = getattr(item, "ssm-range", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.access_list.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Pim-Sm-Global '{value}' not found in "
                    "router/access-list"
                )        
        return errors    
    async def validate_pim_sm_global_vrf_references(self, client: Any) -> list[str]:
        """
        Validate pim_sm_global_vrf references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = MulticastModel(
            ...     pim_sm_global_vrf=[{"bsr-interface": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_pim_sm_global_vrf_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.multicast.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "pim_sm_global_vrf", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("bsr-interface")
            else:
                value = getattr(item, "bsr-interface", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Pim-Sm-Global-Vrf '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/access-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = MulticastModel(
            ...     interface=[{"rpf-nbr-fail-back-filter": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.multicast.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "interface", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("rpf-nbr-fail-back-filter")
            else:
                value = getattr(item, "rpf-nbr-fail-back-filter", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.access_list.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Interface '{value}' not found in "
                    "router/access-list"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_pim_sm_global_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_pim_sm_global_vrf_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "MulticastModel",    "MulticastPimSmGlobal",    "MulticastPimSmGlobal.RpAddress",    "MulticastPimSmGlobalVrf",    "MulticastPimSmGlobalVrf.RpAddress",    "MulticastInterface",    "MulticastInterface.JoinGroup",    "MulticastInterface.Igmp",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:02.498918Z
# ============================================================================


"""
Pydantic Models for CMDB - router/key_chain

Runtime validation models for router/key_chain configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class KeyChainKeyAlgorithmEnum(str, Enum):
    """Allowed values for algorithm field in key."""
    MD5 = "md5"
    HMAC_SHA1 = "hmac-sha1"
    HMAC_SHA256 = "hmac-sha256"
    HMAC_SHA384 = "hmac-sha384"
    HMAC_SHA512 = "hmac-sha512"
    CMAC_AES128 = "cmac-aes128"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class KeyChainKey(BaseModel):
    """
    Child table model for key.
    
    Configuration method to edit key settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: str = Field(max_length=10, serialization_alias="id", description="Key ID (0 - 2147483647).")    
    accept_lifetime: str = Field(description="Lifetime of received authentication key (format: hh:mm:ss day month year).")    
    send_lifetime: str = Field(description="Lifetime of sent authentication key (format: hh:mm:ss day month year).")    
    key_string: Any = Field(max_length=60, description="Password for the key (maximum = 64 characters).")    
    algorithm: KeyChainKeyAlgorithmEnum | None = Field(default=KeyChainKeyAlgorithmEnum.MD5, description="Cryptographic algorithm.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class KeyChainModel(BaseModel):
    """
    Pydantic model for router/key_chain configuration.
    
    Configure key-chain.
    
    Validation Rules:        - name: max_length=35 pattern=        - key: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Key-chain name.")    
    key: list[KeyChainKey] = Field(default_factory=list, description="Configuration method to edit key settings.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "KeyChainModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "KeyChainModel",    "KeyChainKey",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:01.532150Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: switch_controller/traffic_sniffer
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class TrafficSnifferTargetportInportsItem(TypedDict, total=False):
    """Nested item for target-port.in-ports field."""
    name: str


class TrafficSnifferTargetportOutportsItem(TypedDict, total=False):
    """Nested item for target-port.out-ports field."""
    name: str


class TrafficSnifferTargetmacItem(TypedDict, total=False):
    """Nested item for target-mac field."""
    mac: str
    description: str


class TrafficSnifferTargetipItem(TypedDict, total=False):
    """Nested item for target-ip field."""
    ip: str
    description: str


class TrafficSnifferTargetportItem(TypedDict, total=False):
    """Nested item for target-port field."""
    switch_id: str
    description: str
    in_ports: str | list[str] | list[TrafficSnifferTargetportInportsItem]
    out_ports: str | list[str] | list[TrafficSnifferTargetportOutportsItem]


class TrafficSnifferPayload(TypedDict, total=False):
    """Payload type for TrafficSniffer operations."""
    mode: Literal["erspan-auto", "rspan", "none"]
    erspan_ip: str
    target_mac: str | list[str] | list[TrafficSnifferTargetmacItem]
    target_ip: str | list[str] | list[TrafficSnifferTargetipItem]
    target_port: str | list[str] | list[TrafficSnifferTargetportItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class TrafficSnifferResponse(TypedDict, total=False):
    """Response type for TrafficSniffer - use with .dict property for typed dict access."""
    mode: Literal["erspan-auto", "rspan", "none"]
    erspan_ip: str
    target_mac: list[TrafficSnifferTargetmacItem]
    target_ip: list[TrafficSnifferTargetipItem]
    target_port: list[TrafficSnifferTargetportItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class TrafficSnifferTargetportInportsItemObject(FortiObject[TrafficSnifferTargetportInportsItem]):
    """Typed object for target-port.in-ports table items with attribute access."""
    name: str


class TrafficSnifferTargetportOutportsItemObject(FortiObject[TrafficSnifferTargetportOutportsItem]):
    """Typed object for target-port.out-ports table items with attribute access."""
    name: str


class TrafficSnifferTargetmacItemObject(FortiObject[TrafficSnifferTargetmacItem]):
    """Typed object for target-mac table items with attribute access."""
    mac: str
    description: str


class TrafficSnifferTargetipItemObject(FortiObject[TrafficSnifferTargetipItem]):
    """Typed object for target-ip table items with attribute access."""
    ip: str
    description: str


class TrafficSnifferTargetportItemObject(FortiObject[TrafficSnifferTargetportItem]):
    """Typed object for target-port table items with attribute access."""
    switch_id: str
    description: str
    in_ports: FortiObjectList[TrafficSnifferTargetportInportsItemObject]
    out_ports: FortiObjectList[TrafficSnifferTargetportOutportsItemObject]


class TrafficSnifferObject(FortiObject):
    """Typed FortiObject for TrafficSniffer with field access."""
    mode: Literal["erspan-auto", "rspan", "none"]
    erspan_ip: str
    target_mac: FortiObjectList[TrafficSnifferTargetmacItemObject]
    target_ip: FortiObjectList[TrafficSnifferTargetipItemObject]
    target_port: FortiObjectList[TrafficSnifferTargetportItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class TrafficSniffer:
    """
    
    Endpoint: switch_controller/traffic_sniffer
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> TrafficSnifferObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: TrafficSnifferPayload | None = ...,
        mode: Literal["erspan-auto", "rspan", "none"] | None = ...,
        erspan_ip: str | None = ...,
        target_mac: str | list[str] | list[TrafficSnifferTargetmacItem] | None = ...,
        target_ip: str | list[str] | list[TrafficSnifferTargetipItem] | None = ...,
        target_port: str | list[str] | list[TrafficSnifferTargetportItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> TrafficSnifferObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: TrafficSnifferPayload | None = ...,
        mode: Literal["erspan-auto", "rspan", "none"] | None = ...,
        erspan_ip: str | None = ...,
        target_mac: str | list[str] | list[TrafficSnifferTargetmacItem] | None = ...,
        target_ip: str | list[str] | list[TrafficSnifferTargetipItem] | None = ...,
        target_port: str | list[str] | list[TrafficSnifferTargetportItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "TrafficSniffer",
    "TrafficSnifferPayload",
    "TrafficSnifferResponse",
    "TrafficSnifferObject",
]


"""Type stubs for DHCP6 category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .server import Server

__all__ = [
    "Server",
    "Dhcp6",
]


class Dhcp6:
    """DHCP6 API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    server: Server

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize dhcp6 category with HTTP client."""
        ...



"""FortiOS CMDB - SwitchController category"""

from . import fsw_firmware
from . import isl_lockdown
from . import managed_switch
from . import mclag_icl
from . import nac_device
from . import recommendation
from .detected_device import DetectedDevice
from .known_nac_device_criteria_list import KnownNacDeviceCriteriaList
from .matched_devices import MatchedDevices

__all__ = [
    "DetectedDevice",
    "FswFirmware",
    "IslLockdown",
    "KnownNacDeviceCriteriaList",
    "ManagedSwitch",
    "MatchedDevices",
    "MclagIcl",
    "NacDevice",
    "Recommendation",
    "SwitchController",
]


class SwitchController:
    """SwitchController endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """SwitchController endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.fsw_firmware = fsw_firmware.FswFirmware(client)
        self.isl_lockdown = isl_lockdown.IslLockdown(client)
        self.managed_switch = managed_switch.ManagedSwitch(client)
        self.mclag_icl = mclag_icl.MclagIcl(client)
        self.nac_device = nac_device.NacDevice(client)
        self.recommendation = recommendation.Recommendation(client)
        self.detected_device = DetectedDevice(client)
        self.known_nac_device_criteria_list = KnownNacDeviceCriteriaList(client)
        self.matched_devices = MatchedDevices(client)



"""
Pydantic Models for CMDB - firewall/ssl/setting

Runtime validation models for firewall/ssl/setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SettingSslDhBitsEnum(str, Enum):
    """Allowed values for ssl_dh_bits field."""
    V_768 = "768"
    V_1024 = "1024"
    V_1536 = "1536"
    V_2048 = "2048"


# ============================================================================
# Main Model
# ============================================================================

class SettingModel(BaseModel):
    """
    Pydantic model for firewall/ssl/setting configuration.
    
    SSL proxy settings.
    
    Validation Rules:        - proxy_connect_timeout: min=1 max=60 pattern=        - ssl_dh_bits: pattern=        - ssl_send_empty_frags: pattern=        - no_matching_cipher_action: pattern=        - cert_manager_cache_timeout: min=24 max=720 pattern=        - resigned_short_lived_certificate: pattern=        - cert_cache_capacity: min=0 max=500 pattern=        - cert_cache_timeout: min=1 max=120 pattern=        - session_cache_capacity: min=0 max=1000 pattern=        - session_cache_timeout: min=1 max=60 pattern=        - abbreviate_handshake: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    proxy_connect_timeout: int = Field(ge=1, le=60, default=30, description="Time limit to make an internal connection to the appropriate proxy process (1 - 60 sec, default = 30).")    
    ssl_dh_bits: SettingSslDhBitsEnum = Field(default=SettingSslDhBitsEnum.V_2048, description="Bit-size of Diffie-Hellman (DH) prime used in DHE-RSA negotiation (default = 2048).")    
    ssl_send_empty_frags: Literal["enable", "disable"] = Field(default="enable", description="Enable/disable sending empty fragments to avoid attack on CBC IV (for SSL 3.0 and TLS 1.0 only).")    
    no_matching_cipher_action: Literal["bypass", "drop"] = Field(default="bypass", description="Bypass or drop the connection when no matching cipher is found.")    
    cert_manager_cache_timeout: int = Field(ge=24, le=720, default=72, description="Time limit for certificate manager to keep FortiGate re-signed server certificate (24 - 720 hours, default = 72).")    
    resigned_short_lived_certificate: Literal["enable", "disable"] = Field(default="enable", description="Enable/disable short-lived certificate.")    
    cert_cache_capacity: int = Field(ge=0, le=500, default=200, description="Maximum capacity of the host certificate cache (0 - 500, default = 200).")    
    cert_cache_timeout: int = Field(ge=1, le=120, default=10, description="Time limit to keep certificate cache (1 - 120 min, default = 10).")    
    session_cache_capacity: int = Field(ge=0, le=1000, default=500, description="Capacity of the SSL session cache (--Obsolete--) (1 - 1000, default = 500).")    
    session_cache_timeout: int = Field(ge=1, le=60, default=20, description="Time limit to keep SSL session state (1 - 60 min, default = 20).")    
    abbreviate_handshake: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable use of SSL abbreviated handshake.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SettingModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:02.413764Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/replacemsg_group
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ReplacemsgGroupMailItem(TypedDict, total=False):
    """Nested item for mail field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupHttpItem(TypedDict, total=False):
    """Nested item for http field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupWebproxyItem(TypedDict, total=False):
    """Nested item for webproxy field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupFtpItem(TypedDict, total=False):
    """Nested item for ftp field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupFortiguardwfItem(TypedDict, total=False):
    """Nested item for fortiguard-wf field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupSpamItem(TypedDict, total=False):
    """Nested item for spam field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAlertmailItem(TypedDict, total=False):
    """Nested item for alertmail field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAdminItem(TypedDict, total=False):
    """Nested item for admin field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAuthItem(TypedDict, total=False):
    """Nested item for auth field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupSslvpnItem(TypedDict, total=False):
    """Nested item for sslvpn field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupNacquarItem(TypedDict, total=False):
    """Nested item for nac-quar field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupTrafficquotaItem(TypedDict, total=False):
    """Nested item for traffic-quota field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupUtmItem(TypedDict, total=False):
    """Nested item for utm field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupCustommessageItem(TypedDict, total=False):
    """Nested item for custom-message field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupIcapItem(TypedDict, total=False):
    """Nested item for icap field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAutomationItem(TypedDict, total=False):
    """Nested item for automation field."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupPayload(TypedDict, total=False):
    """Payload type for ReplacemsgGroup operations."""
    name: str
    comment: str
    group_type: Literal["default", "utm", "auth"]
    mail: str | list[str] | list[ReplacemsgGroupMailItem]
    http: str | list[str] | list[ReplacemsgGroupHttpItem]
    webproxy: str | list[str] | list[ReplacemsgGroupWebproxyItem]
    ftp: str | list[str] | list[ReplacemsgGroupFtpItem]
    fortiguard_wf: str | list[str] | list[ReplacemsgGroupFortiguardwfItem]
    spam: str | list[str] | list[ReplacemsgGroupSpamItem]
    alertmail: str | list[str] | list[ReplacemsgGroupAlertmailItem]
    admin: str | list[str] | list[ReplacemsgGroupAdminItem]
    auth: str | list[str] | list[ReplacemsgGroupAuthItem]
    sslvpn: str | list[str] | list[ReplacemsgGroupSslvpnItem]
    nac_quar: str | list[str] | list[ReplacemsgGroupNacquarItem]
    traffic_quota: str | list[str] | list[ReplacemsgGroupTrafficquotaItem]
    utm: str | list[str] | list[ReplacemsgGroupUtmItem]
    custom_message: str | list[str] | list[ReplacemsgGroupCustommessageItem]
    icap: str | list[str] | list[ReplacemsgGroupIcapItem]
    automation: str | list[str] | list[ReplacemsgGroupAutomationItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ReplacemsgGroupResponse(TypedDict, total=False):
    """Response type for ReplacemsgGroup - use with .dict property for typed dict access."""
    name: str
    comment: str
    group_type: Literal["default", "utm", "auth"]
    mail: list[ReplacemsgGroupMailItem]
    http: list[ReplacemsgGroupHttpItem]
    webproxy: list[ReplacemsgGroupWebproxyItem]
    ftp: list[ReplacemsgGroupFtpItem]
    fortiguard_wf: list[ReplacemsgGroupFortiguardwfItem]
    spam: list[ReplacemsgGroupSpamItem]
    alertmail: list[ReplacemsgGroupAlertmailItem]
    admin: list[ReplacemsgGroupAdminItem]
    auth: list[ReplacemsgGroupAuthItem]
    sslvpn: list[ReplacemsgGroupSslvpnItem]
    nac_quar: list[ReplacemsgGroupNacquarItem]
    traffic_quota: list[ReplacemsgGroupTrafficquotaItem]
    utm: list[ReplacemsgGroupUtmItem]
    custom_message: list[ReplacemsgGroupCustommessageItem]
    icap: list[ReplacemsgGroupIcapItem]
    automation: list[ReplacemsgGroupAutomationItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ReplacemsgGroupMailItemObject(FortiObject[ReplacemsgGroupMailItem]):
    """Typed object for mail table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupHttpItemObject(FortiObject[ReplacemsgGroupHttpItem]):
    """Typed object for http table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupWebproxyItemObject(FortiObject[ReplacemsgGroupWebproxyItem]):
    """Typed object for webproxy table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupFtpItemObject(FortiObject[ReplacemsgGroupFtpItem]):
    """Typed object for ftp table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupFortiguardwfItemObject(FortiObject[ReplacemsgGroupFortiguardwfItem]):
    """Typed object for fortiguard-wf table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupSpamItemObject(FortiObject[ReplacemsgGroupSpamItem]):
    """Typed object for spam table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAlertmailItemObject(FortiObject[ReplacemsgGroupAlertmailItem]):
    """Typed object for alertmail table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAdminItemObject(FortiObject[ReplacemsgGroupAdminItem]):
    """Typed object for admin table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAuthItemObject(FortiObject[ReplacemsgGroupAuthItem]):
    """Typed object for auth table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupSslvpnItemObject(FortiObject[ReplacemsgGroupSslvpnItem]):
    """Typed object for sslvpn table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupNacquarItemObject(FortiObject[ReplacemsgGroupNacquarItem]):
    """Typed object for nac-quar table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupTrafficquotaItemObject(FortiObject[ReplacemsgGroupTrafficquotaItem]):
    """Typed object for traffic-quota table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupUtmItemObject(FortiObject[ReplacemsgGroupUtmItem]):
    """Typed object for utm table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupCustommessageItemObject(FortiObject[ReplacemsgGroupCustommessageItem]):
    """Typed object for custom-message table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupIcapItemObject(FortiObject[ReplacemsgGroupIcapItem]):
    """Typed object for icap table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupAutomationItemObject(FortiObject[ReplacemsgGroupAutomationItem]):
    """Typed object for automation table items with attribute access."""
    msg_type: str
    buffer: str
    header: Literal["none", "http", "8bit"]
    format: Literal["none", "text", "html"]


class ReplacemsgGroupObject(FortiObject):
    """Typed FortiObject for ReplacemsgGroup with field access."""
    name: str
    comment: str
    group_type: Literal["default", "utm", "auth"]
    mail: FortiObjectList[ReplacemsgGroupMailItemObject]
    http: FortiObjectList[ReplacemsgGroupHttpItemObject]
    webproxy: FortiObjectList[ReplacemsgGroupWebproxyItemObject]
    ftp: FortiObjectList[ReplacemsgGroupFtpItemObject]
    fortiguard_wf: FortiObjectList[ReplacemsgGroupFortiguardwfItemObject]
    spam: FortiObjectList[ReplacemsgGroupSpamItemObject]
    alertmail: FortiObjectList[ReplacemsgGroupAlertmailItemObject]
    admin: FortiObjectList[ReplacemsgGroupAdminItemObject]
    auth: FortiObjectList[ReplacemsgGroupAuthItemObject]
    sslvpn: FortiObjectList[ReplacemsgGroupSslvpnItemObject]
    nac_quar: FortiObjectList[ReplacemsgGroupNacquarItemObject]
    traffic_quota: FortiObjectList[ReplacemsgGroupTrafficquotaItemObject]
    utm: FortiObjectList[ReplacemsgGroupUtmItemObject]
    custom_message: FortiObjectList[ReplacemsgGroupCustommessageItemObject]
    icap: FortiObjectList[ReplacemsgGroupIcapItemObject]
    automation: FortiObjectList[ReplacemsgGroupAutomationItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class ReplacemsgGroup:
    """
    
    Endpoint: system/replacemsg_group
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ReplacemsgGroupObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ReplacemsgGroupObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ReplacemsgGroupPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        group_type: Literal["default", "utm", "auth"] | None = ...,
        mail: str | list[str] | list[ReplacemsgGroupMailItem] | None = ...,
        http: str | list[str] | list[ReplacemsgGroupHttpItem] | None = ...,
        webproxy: str | list[str] | list[ReplacemsgGroupWebproxyItem] | None = ...,
        ftp: str | list[str] | list[ReplacemsgGroupFtpItem] | None = ...,
        fortiguard_wf: str | list[str] | list[ReplacemsgGroupFortiguardwfItem] | None = ...,
        spam: str | list[str] | list[ReplacemsgGroupSpamItem] | None = ...,
        alertmail: str | list[str] | list[ReplacemsgGroupAlertmailItem] | None = ...,
        admin: str | list[str] | list[ReplacemsgGroupAdminItem] | None = ...,
        auth: str | list[str] | list[ReplacemsgGroupAuthItem] | None = ...,
        sslvpn: str | list[str] | list[ReplacemsgGroupSslvpnItem] | None = ...,
        nac_quar: str | list[str] | list[ReplacemsgGroupNacquarItem] | None = ...,
        traffic_quota: str | list[str] | list[ReplacemsgGroupTrafficquotaItem] | None = ...,
        utm: str | list[str] | list[ReplacemsgGroupUtmItem] | None = ...,
        custom_message: str | list[str] | list[ReplacemsgGroupCustommessageItem] | None = ...,
        icap: str | list[str] | list[ReplacemsgGroupIcapItem] | None = ...,
        automation: str | list[str] | list[ReplacemsgGroupAutomationItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ReplacemsgGroupObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ReplacemsgGroupPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        group_type: Literal["default", "utm", "auth"] | None = ...,
        mail: str | list[str] | list[ReplacemsgGroupMailItem] | None = ...,
        http: str | list[str] | list[ReplacemsgGroupHttpItem] | None = ...,
        webproxy: str | list[str] | list[ReplacemsgGroupWebproxyItem] | None = ...,
        ftp: str | list[str] | list[ReplacemsgGroupFtpItem] | None = ...,
        fortiguard_wf: str | list[str] | list[ReplacemsgGroupFortiguardwfItem] | None = ...,
        spam: str | list[str] | list[ReplacemsgGroupSpamItem] | None = ...,
        alertmail: str | list[str] | list[ReplacemsgGroupAlertmailItem] | None = ...,
        admin: str | list[str] | list[ReplacemsgGroupAdminItem] | None = ...,
        auth: str | list[str] | list[ReplacemsgGroupAuthItem] | None = ...,
        sslvpn: str | list[str] | list[ReplacemsgGroupSslvpnItem] | None = ...,
        nac_quar: str | list[str] | list[ReplacemsgGroupNacquarItem] | None = ...,
        traffic_quota: str | list[str] | list[ReplacemsgGroupTrafficquotaItem] | None = ...,
        utm: str | list[str] | list[ReplacemsgGroupUtmItem] | None = ...,
        custom_message: str | list[str] | list[ReplacemsgGroupCustommessageItem] | None = ...,
        icap: str | list[str] | list[ReplacemsgGroupIcapItem] | None = ...,
        automation: str | list[str] | list[ReplacemsgGroupAutomationItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ReplacemsgGroupObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ReplacemsgGroupPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        group_type: Literal["default", "utm", "auth"] | None = ...,
        mail: str | list[str] | list[ReplacemsgGroupMailItem] | None = ...,
        http: str | list[str] | list[ReplacemsgGroupHttpItem] | None = ...,
        webproxy: str | list[str] | list[ReplacemsgGroupWebproxyItem] | None = ...,
        ftp: str | list[str] | list[ReplacemsgGroupFtpItem] | None = ...,
        fortiguard_wf: str | list[str] | list[ReplacemsgGroupFortiguardwfItem] | None = ...,
        spam: str | list[str] | list[ReplacemsgGroupSpamItem] | None = ...,
        alertmail: str | list[str] | list[ReplacemsgGroupAlertmailItem] | None = ...,
        admin: str | list[str] | list[ReplacemsgGroupAdminItem] | None = ...,
        auth: str | list[str] | list[ReplacemsgGroupAuthItem] | None = ...,
        sslvpn: str | list[str] | list[ReplacemsgGroupSslvpnItem] | None = ...,
        nac_quar: str | list[str] | list[ReplacemsgGroupNacquarItem] | None = ...,
        traffic_quota: str | list[str] | list[ReplacemsgGroupTrafficquotaItem] | None = ...,
        utm: str | list[str] | list[ReplacemsgGroupUtmItem] | None = ...,
        custom_message: str | list[str] | list[ReplacemsgGroupCustommessageItem] | None = ...,
        icap: str | list[str] | list[ReplacemsgGroupIcapItem] | None = ...,
        automation: str | list[str] | list[ReplacemsgGroupAutomationItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "ReplacemsgGroup",
    "ReplacemsgGroupPayload",
    "ReplacemsgGroupResponse",
    "ReplacemsgGroupObject",
]